@@ -20,6 +20,7 @@ import interfaces
 from Products.ATContentTypes.content.folder import ATFolder
 from Products.CMFDynamicViewFTI.browserdefault import BrowserDefaultMixin
 
+from Products.CMFPlomino import plomino_profiler
 from Products.CMFPlomino.config import *
 from Products.CMFPlomino.PlominoUtils import PlominoTranslate, DateToString
 from Products.CMFPlomino.exceptions import PlominoDesignException
@@ -390,6 +391,7 @@ class PlominoForm(ATFolder):
 
 
     security.declareProtected(READ_PERMISSION, 'displayDocument')
+    @plomino_profiler('form')
     def displayDocument(self, doc, editmode=False, creation=False, parent_form_id=False, request=None):
         """display the document using the form's layout
         """
@@ -425,15 +427,18 @@ class PlominoForm(ATFolder):
         for (field, fieldblock) in fields_in_layout:
             # check if fieldblock still here after cache replace
             if fieldblock in html_content:
-                html_content = html_content.replace(fieldblock, field.getFieldRender(self, doc, editmode, creation, request=request))
+                html_content = html_content.replace(
+                        fieldblock,
+                        field.getFieldRender(self, doc, editmode, creation,
+                                request=request))
 
         # insert subforms
         for subformname in self.getSubforms(doc):
             subform = self.getParentDatabase().getForm(subformname)
             if subform:
                 subformrendering = subform.displayDocument(
-                    doc, editmode, creation, parent_form_id=self.id,
-                    request=request)
+                        doc, editmode, creation, parent_form_id=self.id,
+                        request=request)
                 html_content = html_content.replace('<span class="plominoSubformClass">'+subformname+'</span>', subformrendering)
 
         # insert the actions
@@ -762,7 +767,7 @@ class PlominoForm(ATFolder):
             except PlominoScriptException, e:
                 if self.REQUEST:
                     e.reportError('Search event failed.')
-                    self.OpenForm(searchresults=[])
+                    return self.OpenForm(searchresults=[])
         else:
             # Allow Plomino to filter by view, default query, and formula
             db = self.getParentDatabase()
@@ -792,7 +797,10 @@ class PlominoForm(ATFolder):
                         if fieldname == "Plomino_SearchableText":
                             fieldname = "SearchableText"
                         query[fieldname]=v
-            results=index.dbsearch(query, None)
+            sortindex = searchview.getSortColumn()
+            if not sortindex:
+                sortindex = None
+            results=index.dbsearch(query, sortindex=sortindex, reverse=searchview.getReverseSorting())
 
             #filter search with searchformula
             searchformula=self.getSearchFormula()
@@ -836,9 +844,9 @@ class PlominoForm(ATFolder):
                     if fieldtype == "ATTACHMENT" and doc:
                         existing_files = doc.getItem(fieldname)
                         if not existing_files:
-                            errors.append(fieldname+" "+PlominoTranslate("is mandatory",self))
+                            errors.append(f.Title()+" "+PlominoTranslate("is mandatory",self))
                     else:
-                        errors.append(fieldname+" "+PlominoTranslate("is mandatory",self))
+                        errors.append(f.Title()+" "+PlominoTranslate("is mandatory",self))
             else:
                 # STEP 2: check data types
                 errors = errors + f.validateFormat(submittedValue)
