@@ -36,6 +36,26 @@ except ImportError:
 import logging
 logger = logging.getLogger('Plomino')
 
+severity_map = {
+        'info': logging.INFO,
+        'warning': logging.WARNING,
+        'debug': logging.DEBUG
+        }
+
+
+def Log(message, summary='', severity='info', exc_info=False):
+    """ Write a message to the event log. Useful from formulas.
+    Optionally include a traceback.
+    """
+    # Pass in severity as a string, because we don't want to import
+    # ``logging`` from scripts.
+    logger.log(
+            severity_map.get(severity, 'info'),
+            'App: %s\n%s',
+            summary,
+            message,
+            exc_info=exc_info)
+
 def DateToString(d, format='%Y-%m-%d'):
     """ Return the date as string using the given format
     """
@@ -52,17 +72,18 @@ def StringToDate(str_d, format='%Y-%m-%d'):
         # XXX: Just let DateTime guess.
         dt = strptime(DateTime(str_d).ISO(), '%Y-%m-%d %H:%M:%S')
         logger.info('StringToDate> %s, %s, %s, guessed: %s'%(str(str_d), format, `e`, `dt`))
-    if len(dt)>=5:
+    if len(dt) >= 5:
         return DateTime(dt[0], dt[1], dt[2], dt[3], dt[4])
     else:
         return DateTime(dt[0], dt[1], dt[2])
 
 def DateRange(d1, d2):
-    """return all the days from the d1 date to the d2 date (included)
+    """ Return all the dates from ``d1`` to ``d2`` (inclusive).
+    Dates are ``DateTime`` instances.
     """
     duration = int(d2-d1)
-    result=[]
-    current=d1
+    result = []
+    current = d1
     for d in range(duration+1):
         result.append(current)
         current = current+1
@@ -99,85 +120,98 @@ def sendMail(db, recipients, title, html_message, sender=None, cc=None, bcc=None
         host.secureSend(message, recipients,
                         sender, subject=title,
                         subtype='html', charset='utf-8')
-        
+
+
 def userFullname(db, userid):
-    """ return user fullname if exist, else return userid, and return Unknown if user not found
+    """ Try to return user fullname, else return userid.
+    Return "Unknown" if user not found.
     """
-    user=getToolByName(db, 'portal_membership').getMemberById(userid)
-    if not(user is None):
-        fullname=user.getProperty('fullname')
-        if fullname=='':
-            return userid
-        else:
-            return fullname
-    else:
+    user = getToolByName(db, 'portal_membership').getMemberById(userid)
+
+    if not user:
         return "Unknown"
 
-def userInfo(db, userid):
-    """ return user object
-    """
-    user=getToolByName(db, 'portal_membership').getMemberById(userid)
-    return user
+    fullname = user.getProperty('fullname')
+    if fullname:
+        return fullname
+    else:
+        return userid
 
-def PlominoTranslate(message, context, domain='CMFPlomino'):
+
+def userInfo(db, userid):
+    """ Return user object.
     """
+    return getToolByName(db, 'portal_membership').getMemberById(userid)
+
+
+def PlominoTranslate(msgid, context, domain='CMFPlomino'):
+    """ Look up the translation for ``msgid`` in the current language.
     """
     plone_tools = getToolByName(context, 'plone_utils')
     encoding = plone_tools.getSiteEncoding()
     translation_service = getToolByName(context, 'translation_service')
-    if isinstance(message, Exception):
+    # When will message be an exception?
+    if isinstance(msgid, Exception):
         try:
-            message = message[0]
+            msgid = msgid[0]
         except (TypeError, IndexError):
             pass
     if HAS_PLONE40:
-        msg = translation_service.utranslate(domain=domain, msgid=message, context=context)
+        msg = translation_service.utranslate(domain=domain, msgid=msgid, context=context)
     else:
-        msg = translation_service.utranslate(msgid=message, domain=domain, context=context)
+        msg = translation_service.utranslate(msgid=msgid, domain=domain, context=context)
     return translation_service.encode(msg) # convert unicode to site encoding
 
 def htmlencode(s):
-    """ replace unicode characters with their corresponding html entities
+    """ Replace characters with their corresponding HTML entities.
     """
-
-    t=""
-    for i in s:
-        if ord(i) in entity.codepoint2name:
-            name = entity.codepoint2name.get(ord(i))
-            entityCode = entity.name2codepoint.get(name)
-            t +="&#" + str(entityCode)
+    t = ""
+    if type(s) != unicode:
+        from Products.CMFPlone.utils import safe_unicode
+        s = safe_unicode(s)
+        # Doesn't work unless utils becomes a persistent tool.
+        # translation_service = getToolByName(context, 'translation_service')
+        # s = translation_service.asunicodetype(s)
+    for c in s:
+        name = entity.codepoint2name.get(ord(c))
+        if name:
+            t += "&%s;" % name
         else:
-            t+=i
+            t += c
     return t
 
 def urlencode(h):
-    """ call urllib.urlencode
+    """ Call urllib.urlencode
     """
     return urllib.urlencode(h)
 
 def asList(x):
-    """ if not list, return x in a single-element list
-    XXX: Don't call asList if x may be None, as [None] probably is not what you
-    want.
+    """ If not list, return x in a single-element list.
+    .. note:: If ``x`` is ``None``, this will return ``[None]``.
     """
     if hasattr(x, 'append'):
         return x
     else:
         return [x]
 
-def asUnicode(x):
+
+def asUnicode(s):
+    """ Make sure ``s`` is unicode; encode according to site encoding if
+    needed.
     """
-    """
-    if type(x) is str:
-        return x.decode('utf-8')
-    if type(x) is unicode:
-        return x
-    return unicode(x)
+    if not isinstance(s, basestring):
+        return unicode(s)
+    from Products.CMFPlone.utils import safe_unicode
+    return safe_unicode(s)
+    # Doesn't work unless utils becomes a persistent tool.
+    # translation_service = getToolByName(context, 'translation_service')
+    # return translation_service.asunicodetype(s)
+
 
 def csv_to_array(csvcontent, delimiter='\t', quotechar='"'):
-    """ csvcontent might be a string or a file
+    """ ``csvcontent`` may be a string or a file.
     """
-    if csvcontent is None:
+    if not csvcontent:
         return []
     if type(csvcontent) is str:
         csvfile = StringIO(csvcontent)
@@ -185,27 +219,30 @@ def csv_to_array(csvcontent, delimiter='\t', quotechar='"'):
         csvfile = csvcontent.blob.open()
     else:
         csvfile = csvcontent
+    # XXX: why not just return the list-like ``_csv.reader`` object?
     return [l for l in csv.reader(csvfile, delimiter=delimiter, quotechar=quotechar)]
 
-def array_to_csv(values, delimiter='\t', quotechar='"'):
-    """
+
+def array_to_csv(array, delimiter='\t', quotechar='"'):
+    """ Convert ``array`` (a list of lists) to a CSV string.
     """
     s = StringIO()
     writer = csv.writer(s, delimiter=delimiter, quotechar=quotechar, quoting=csv.QUOTE_NONNUMERIC)
-    writer.writerows(values)
+    writer.writerows(array)
     return s.getvalue()
 
+
 def open_url(url, asFile=False):
-    """ retrieve content from url
+    """ Retrieve content from ``url``.
     """
-    f=urllib.urlopen(url)
+    f = urllib.urlopen(url)
     if asFile:
         return f.fp
     else:
         return f.read()
 
 def MissingValue():
-    """ Useful to test search results value (as Missing.Value cannot be imported in scripts)
+    """ Useful to test search results value (as ``Missing.Value`` cannot be imported in scripts).
     """
     return Missing.Value
 
@@ -222,4 +259,4 @@ def json_dumps(obj):
     return json.dumps(obj)
 
 def json_loads(json_string):
-    return json.loads(json_string)
\ No newline at end of file
+    return json.loads(json_string)
